import hell2;
import token;
import player;
import enemy;
import particle;

/**
 * 敵トークン
 */
class Enemy : Token
{
public:
	/**
	 * 親
	 */
	static TokenManager!(Enemy) parent;
	/**
	 * ID
	 */
	int id;
	/**
	 * タイマ
	 */
	int timer;
public:
	static int destroyAll()
	{
		int ret;
		foreach(enemy; Enemy.parent.getArray())
		{
			if(!enemy.exist) { continue; }
			ret += enemy.getScore();
			enemy.vanish();
		}
		return ret;
	}
	int getScore()
	{
		return 100;
	}
	/**
	 * 生成
	 */
	void create(int id, float x, float y, float w, float h, float deg=0, float speed=0)
	{
		super.create(x, y, w, h, deg, speed);
		this.id = id;
		timer   = Hell_randInt(0, 360);
		switch(id)
		{
		case 2:
			vx = Hell_rand(2) == 1 ? -1 : 1;
			vy = Hell_rand(2) == 1 ? -1 : 1;
			break;
		case 4:
			vx = Hell_rand(2) == 1 ? -2 : 2;
			vy = Hell_rand(2) == 1 ? -2 : 2;
			break;
		default:
			break;
		}
	}
	/**
	 * 生成（外から呼ばれる）
	 */
	static void add(int id, float x, float y, float w, float h, float deg=0, float speed=0)
	{
		Enemy enemy = parent.getToken();
		if(!enemy) return;
		enemy.create(id, x, y, w, h, deg, speed);
	}
	static void addEx(int id)
	{
		add(id, Hell_randInt(32, 640-32), Hell_randInt(80, 480-32), 32, 32);
	}
	/**
	 * 消滅
	 */
	void vanish()
	{
		exist = false;
		Particle.addDestroy(x, y, w);
	}
	
	/**
	 * 更新
	 */
	void update()
	{
		timer++;
		switch(id)
		{
		case 3:
			if(timer%120 < 60)
			{
				ax = 0;
				vx *= 0.9f;
				if(y < 240) { ay += 0.005f; }
				else        { ay -= 0.005f; }
			}
			else
			{
				ay = 0;
				vy *= 0.9f;
				if(x < 320) { ax += 0.005f; }
				else        { ax -= 0.005f; }
			}
			break;
		case 5:
			if(timer%120 < 60)
			{
				vx = 5 * Hell_cosEx(timer);
				vy = 5 * -Hell_sinEx(timer);
			}
			else
			{
				vx *= 0.9f;
				vy *= 0.9f;
			}
			break;
			
		default:
			break;
		}
		super.move();
		if(x < w)       { x = w; vx = -vx; }
		if(y < 80+ h)   { y = 80+h; vy = -vy; }
		if(x > 640 - w) { x = 640-w; vx = -vx; }
		if(y > 480 - h) { y = 480-h; vy = -vy; }
	}
	/**
	 * 描画
	 */
	void draw()
	{
		Draw(this, x, y, 1.0f);
	}
	static void Draw(Enemy e, float x, float y, float scale)
	{
		float s = e.w / 64.0f * scale;
		switch(e.id)
		{
		case 1:
			Hell_drawTextureEx("all", x, y, 0, 128, 128, 128, s, s, e.timer);
			break;
		case 2:
			Hell_drawTextureEx("all", x, y, 128, 128*3, 128, 128, s, s, e.timer);
			break;
		case 3:
			Hell_drawTextureEx("all", x, y, 128, 128, 128, 128, s, s, e.timer);
			break;
		case 4:
			Hell_drawTextureEx("all", x, y, 0, 128*3, 128, 128, s, s, e.timer);
			break;
		case 5:
			Hell_drawTextureEx("all", x, y, 128, 128*2, 128, 128, s, s, e.timer);
			break;
			
		}
	}
}

/**
 * 敵弾クラス
 */
class Bullet : Token
{
public:
	/**
	 * 親トークン管理
	 */
	static TokenManager!(Bullet) parent;
	int id;     // ID
public:
	/**
	 * 生成
	 */
	void create(int id, float x, float y, float w, float h, float deg, float speed, float ax=0, float ay=0)
	{
		this.id = id;
		super.create(x, y, w, h, deg, speed, ax, ay);
	}
	/**
	 * 生成（外から呼ばれる）
	 */
	static void add(float x, float y, float deg, float speed, float ax=0, float ay=0)
	{
		Bullet bullet = parent.getToken();
		if(!bullet) return;
		bullet.create(0, x, y, 8, 8, deg, speed, ax, ay);
	}
	/**
	 * 更新
	 */
	void update()
	{
		super.move();
		if(isOutOfWindow())
		{
			exist = false;
		}
	}
	/**
	 * 描画
	 */
	void draw()
	{
	}
	void vanish()
	{
		Particle.addBullet(x, y, deg, speed);
		exist = false;
	}
	static void vanishAll()
	{
		foreach(bullet; Bullet.parent.getArray())
		{
			if(!bullet.exist) { continue; }
			bullet.vanish();
		}
	}
}
