import hell2;

class IToken
{
public:
	static int screenW;
	static int screenH;
	static void setScreen(int w, int h)
	{
		screenW = w;
		screenH = h;
	}
}

/**
 * トークンクラス
 */
class Token : IToken
{
public:
	
	/**
	 * 座標
	 */
	float x, y;
	/**
	 * 移動量
	 */
	float vx, vy;
	/**
	 * 加速度
	 */
	float ax, ay;
	float deg;
	float speed;
	/**
	 * 生存フラグ
	 */
	bool exist;
	/**
	 * サイズ
	 */
	float w, h;
public:
	/**
	 * コンストラクタ
	 */
	this()
	{
		exist = false;
	}
	/**
	 * 生成
	 */
	void create(float x, float y, float w, float h, float deg=0, float speed=0, float ax=0, float ay=0)
	{
		this.x = x; this.y = y;
		this.w = w; this.h = h;
		this.deg   = deg;
		this.speed = speed;
		this.vx = Hell_cosEx(deg)  * speed;
		this.vy = -Hell_sinEx(deg) * speed;
		this.ax = ax; this.ay = ay;
		exist = true;
	}
	/**
	 * 移動
	 */
	void move()
	{
		vx += ax; vy += ay;
		x += vx;  y += vy;
	}
	/**
	 * 点との当たり判定
	 */
	bool isHit(int px, int py)
	{
		float left   = x - w;
		float top    = y - h;
		float right  = x + w;
		float bottom = y + h;
		return left <= px && px <= right && top <= py && py <= bottom;
	}
	/**
	 * 別のトークン（矩形）との当たり判定
	 */
	bool isHit(Token t)
	{
		float left1   = x - w;
		float top1    = y - h;
		float right1  = x + w;
		float bottom1 = y + h;
		float left2   = t.x - t.w;
		float top2    = t.y - t.h;
		float right2  = t.x + t.w;
		float bottom2 = t.y + t.h;
		return (left1 < right2) && (right1 > left2) && (top1 < bottom2) && (bottom1 > top2);
	}
	/**
	 * 別のトークン（円）との当たり判定
	 */
	bool isHit2(Token t)
	{
		float dx = x - t.x;
		float dy = y - t.y;
		return (dx*dx + dy*dy) < (w + t.w) * (w + t.w);
	}
	/**
	 * ウィンドウサイズ内に存在するかどうか
	 */
	bool isOutOfWindow()
	{
		return x < -w || x > screenW+w || y < -h || y > screenH+h;
	}
	void dump()
	{
		Hell_drawFont(
			std.string.toString(x) ~ "," ~ std.string.toString(y) ~ "," ~ std.string.toString(vx) ~ "," ~ std.string.toString(vy),
			0, 0
		);
	}
	/**
	 * 描画
	 */
	void draw(int r, int g, int b)
	{
		Hell_drawRect(x, y, w, h, 0, 0, r, g, b);
	}
}

/**
 * トークンの管理クラス
 */
class TokenManager(T)
{
public:
	/**
	 * 保持しているインスタンス
	 */
	T[] _pool;
public:
	/**
	 * コンストラクタ
	 */
	this(int size)
	{
		_pool = new T[size];
		foreach(inout t; _pool)
		{
			t = new T;
		}
	}
	/**
	 * 利用可能なトークンの取得
	 */
	T getToken()
	{
		foreach(t; _pool)
		{
			if(!t.exist) return t;
		}
		return null;
	}
	/**
	 * 保持しているトークン配列の取得
	 */
	T[] getArray() { return _pool; }
	
	/**
	 * 生存数を取得する
	 */
	int count()
	{
		int ret = 0;
		foreach(t; _pool)
		{
			if(t.exist) { ret++; }
		}
		return ret;
	}
	/**
	 * 更新
	 */
	void update()
	{
		foreach(t; _pool)
		{
			if(!t.exist) continue;
			t.update();
		}
	}
	/**
	 * 描画
	 */
	void draw()
	{
		foreach(t; _pool)
		{
			if(!t.exist) continue;
			t.draw();
		}
	}
	/**
	 * 全てを消滅
	 */
	void vanish()
	{
		foreach(t; _pool)
		{
			t.exist = false;
		}
	}
}

/**
 * ２次元ベクトル
 */
class Vec2D
{
public:
	/**
	 * 値
	 */
	float x, y;
	/**
	 * コンストラクタ
	 */
	this(float x=0, float y=0) { this.x = x; this.y = y; }
	void opAddAssign(Vec2D v)  { x += v.x; y += v.y; }
	void opSubAssign(Vec2D v)  { x -= v.x; y -= v.y; }
	void opMulAssign(float a)  { x *= a;   y *= a;   }
	void opDivAssign(float a)  { x /= a;   y /= a;   }
	/**
	 * 正規化
	 */
	void normalize()
	{
		float len = length();
		if(len == 0) return;
		x /= len; y /= len;
	}
	/**
	 * 距離
	 */
	float length()   { return Hell_sqrt(lengthSq());}
	/**
	 * 距離の２乗
	 */
	float lengthSq() { return x*x + y*y; }
}

/**
 * ゲームデータ管理
 */
class Game
{
public:
	static int score;
	static int hiscore;
public:
	static void addScore(int val)
	{
		score += val;
		if(score > hiscore)
		{
			hiscore = score;
		}
	}
}

/**
 * シーン基底クラス
 */
class IScene : IToken
{
public:
	static Game game;
public:
	void init()   {}
	void update() {}
}

/**
 * シーン管理
 */
class SceneManager
{
public:
	static IScene pool[char[]]; // インスタンス管理
	static IScene current;
public:
	static void regist(char[] key, IScene s)
	{
		pool[key] = s;
		current   = s;
	}
	static void change(char[] key)
	{
		current = pool[key];
		Hell_write("Scene Chanage => %s\n", key);
	}
	static IScene get()
	{
		return current;
	}
	static void update()
	{
		current.update();
	}
}
